#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmProductsHelper

  # サマリー表のチェックボックス
  def create_summary_product_chkbox(prod_id_info, checked)
    content = check_box_tag(
      'table[selected]',
      prod_id_info.to_json,
      checked.include?(prod_id_info),
      :class => 'SelectProduct',
    )
  end

  # サマリー表のプロダクト名
  def create_summary_product_name(prod)
    link_to(
      prod[:name],
      {
        :controller => :sidfm_informations,
        :action => :product,
        :id => (prod[:product_id] ? prod[:product_id] : 'lproduct' + prod[:lproduct_id].to_s)
      }
    )
  end

  # サマリー表のバージョン名
  def create_summary_product_version_name(version_name)
    content = content_tag(:div, "#{version_name}")
  end

  #サマリー表のホスト
  def create_summary_product_hosts(hosts)
    content =
      if hosts.size > 1 then
        create_hosts_balloon(hosts)
      elsif hosts.size == 1 then
        content_tag(
          :span,
          :class => "d-block text-truncate g-max-width-12em",
          :data => {
            :toggle => "tooltip",
            :html => "true",
            :title => "ホスト名: #{hosts.first[:hostname]}<br/>表示名: #{hosts.first[:ext_name] ? hosts.first[:ext_name] : "(未設定)"}",
            :placement => "right",
            :boundary => "window"
          }
        ) do
          link_to(
            hosts.first[:display_name],
            '#',#{ :controller => :sidfm_products, :action => :index },
            :class => "search-host-by-display-name",
            :data => { :displayname => hosts.first[:display_name] }
          )
        end
      else
        "---"
      end
  end

  # サマリー表のSIDfmサポート
  def create_summary_product_is_support_end(is_support_end)
    content = content_tag(:i, nil, :class => "#{is_support_end ? "fas fa-times" : "far fa-circle"} g-font-size-1_16x")
  end

  # サマリー表のリンク
  def create_summary_product_link_to_pages(prod)
    filter_products_param = {}
      if prod[:pid] then
        filter_products_param[:product_ids] = [ prod[:pid] ]
      elsif prod[:vid] then
        filter_products_param[:version_ids] = [ prod[:vid] ]
      elsif prod[:lpid] then
        filter_products_param[:lproduct_ids] = [ prod[:lpid] ]
      elsif prod[:lvid] then
        filter_products_param[:lversion_ids] = [ prod[:lvid] ]
      end
    hosts_filter_products_param = {}
    [ :product_all_version_ids , :product_ids, 
      :version_ids, :lproduct_all_version_ids, :lproduct_ids, :lversion_ids
    ].each do |filter_key|
      hosts_filter_products_param[filter_key] = filter_products_param[filter_key] || []
    end

    vuls_filter_products_param = {}
    [ :product_ids, :version_ids, :lproduct_ids, :lversion_ids ].each do |filter_key|
      vuls_filter_products_param[filter_key] = filter_products_param[filter_key] || []
    end 
  

    content = content_tag(
      :div
    ) do
      concat(
        link_to(
          'ホスト別状況',
          {
            :controller => :sidfm_hosts,
            :action => :index_r,
            :'pages[filters]' => { 'products' => hosts_filter_products_param }.to_json,
          },
          :class => 'text-truncate g-text-ul g-letter-spacing-0_08'
        )
      )
    end
    .concat(
      content_tag(
        :div
      ) do
        concat(
          link_to(
            '脆弱性別状況',
            {
              :controller => :sidfm_vulnerabilities,
              :action => :index_r,
              :'pages[filters]' => { 'products' => vuls_filter_products_param }.to_json,
            },
            :class => 'text-truncate g-text-ul g-letter-spacing-0_08'
          )
        )
      end
    )
  end

end
