#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmRuleSettingHelper

  # サマリー表のチェックボックス
  def create_summary_rule_chkbox(rule_id, is_user_admin, checked)
    content = check_box_tag(
      nil,
      rule_id,
      checked.include?(rule_id),
      :class => 'SelectRule',
      :disabled => is_user_admin
    )
  end
  
  # サマリー表のルール名
  def create_summary_rule_name(rule_name)
    content = content_tag(:div, rule_name, :class => "word-break-all g-text-shadow-eee")
  end

  # サマリー表のカテゴリ
  def create_summary_rule_category(rule_category_name, rule_category_id)
    target_cat = nil
    if rule_category_id != "all" then
      target_cat = SidfmdCategory.find_by(:id => rule_category_id)
    end
    content = content_tag(
      :div,
      rule_category_name,
      :class => "word-break-all",
      :data => { :toggle => "tooltip", :title => "#{target_cat ? target_cat.desc : ""}", :placement => "right", :boundary => "window" }
    )
  end

  # サマリー表のタイミング
  def create_summary_rule_trigger(trigger)
    content = content_tag(
      :div,
      :class => "g-brd-around g-brd-ccc g-rounded-3 mb-0"
    ) do
      concat(
        content_tag(:span, trigger[0], :class => "g-text-shadow-eee")
      )
    end
    .concat(
      content_tag(
        :div
      ) do
        concat(
          content_tag(
            :span,
            "詳細",
            :class => "badge badge-info g-font-size-10 g-letter-spacing-0_08 font-weight-normal cursor-pointer text-nowrap",
            :data => { :toggle => "tooltip", :title => "#{trigger[1]}", :placement => "left" }
          )
        )
      end
    )
  end

  # サマリー表の条件
  def create_summary_rule_condition(rule)
    content = button_tag(
      :type => "button",
      :class => "btn btn-xs btn-light-nb text-nowrap p-1 open-RuleTable-detail show-original-popover",
      :data => { :detail => "cond", :ruleid => "#{rule[:rule_id]}" }
    ) do
      concat("条件")
      .concat(
        content_tag(:span, "#{rule[:cond_num]}", :class => "badge badge-pill badge-gray-999 rule-num-mark")
      )
    end
  end

  # サマリー表の操作
  def create_summary_rule_operation(rule)
    content = button_tag(
      :type => "button",
      :class => "btn btn-xs btn-light-nb text-nowrap p-1 open-RuleTable-detail show-original-popover",
      :data => { :detail => "ope", :ruleid => "#{rule[:rule_id]}" }
    ) do
      concat("操作")
      .concat(content_tag(:span, "#{rule[:ope_num]}", :class => "badge badge-pill badge-gray-999 rule-num-mark"))
    end
  end

  # サマリー表の優先度
  def create_summary_rule_priority(priority)
    content_tag(:span, priority)
  end

  # サマリー表のログ
  def create_summary_rule_log(rule_name, logs)
    content = logs.any? ? 
      button_tag(
        :type => "button",
        :class => 'btn btn-xs btn-light-nb text-nowrap p-1 open-logs-list',
        :data => {
          :log => logs.to_json,
          :rulename => rule_name
        }
      ) do
        concat("ログ")
        .concat(
          content_tag(:span, "#{logs.size}", :class => "badge badge-pill badge-gray-999 ml-1 rule-num-mark")
        )
      end :
      "無"
  end

  # サマリー表の有効無効
  def create_summary_rule_status(is_enabled)
    content = is_enabled ?
      content_tag(:span, "有効", :class => "badge badge-success g-font-size-0_93x font-weight-normal text-nowrap") :
      content_tag(:span, "無効", :class => "badge badge-secondary g-font-size-0_93x font-weight-normal text-nowrap")
  end

  # サマリー表の編集
  def create_summary_rule_edit(rule_id, rule_name)
    content = content_tag(
      :div,
      :class => "d-flex justify-content-center"
    ) do
      concat(
        button_tag(
          "編集",
          :type => "button",
          :class => "mr-1 btn btn-sm btn-edit edit-rule text-nowrap",
          :data => { :edit => "rule-setting", :ruleid => "#{rule_id}" },
        )
      )
      .concat(
        form_tag(
          { :action => :delete },
          :method => :post,
          :id => "delete-rule-form-#{rule_id}"
        ) do
          concat(
            button_tag(
              "削除",
              :type => "button",
              :class => "btn btn-sm btn-danger delete-rule text-nowrap",
              :data => { :confirm => "「#{rule_name}」を削除してもよろしいですか?", :ruleid => "#{rule_id}" },
            )
          )
          .concat( hidden_field_tag("form[delete_id]", "#{rule_id}") )
        end
      )
    end

  end

end
