#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdCategoriesHost < ActiveRecord::Base
  belongs_to :sidfmd_category
  belongs_to :sidfmd_host

  scope :categories, -> (host){ where(:sidfmd_host => host) }
  scope :hosts, -> (category){ where(:sidfmd_category => category) }

  scope :system_categories, ->{ joins(:sidfmd_category).where('sidfmd_category_id IN (?)', SidfmdCategory.system.select(:id).to_a) }
  scope :enable_hosts, ->{ joins(:sidfmd_host).where(:sidfmd_hosts => { :flg_delete => false }) }
  scope :enable_category_hosts, -> { where(:sidfmd_category_id => SidfmdCategory.system.select(:id).to_a) }

  # カテゴリ毎のホスト数を返す。
  def self.distribution_host(category_id = 0)
    cnt = SidfmdCategoriesHost.enable_hosts.enable_category_hosts.system_categories
    if category_id > 0 then
      cnt = cnt.where(:sidfmd_category_id => category_id)
    end
    dist = cnt.group(:sidfmd_category_id).count
    ret = {}
    [ :public_server, :private_server, :mobile, :client, :shared_server,
      :virtual_host_server, :public_network,
      :private_network ].each_with_index do |k, i|
      ret[k] = dist[i + 1] ? dist[i + 1] : 0
    end
    ret
  end

  # ホストに登録する(紐づける)カテゴリのうち、hostlimの値が登録済みホスト数を超えているカテゴリを返す
  # host_categoryidsは、{ ホストid: 登録するカテゴリid }のハッシュにする
  def self.hostlim_over_categories(host_categoryids)
    # 複数ホスト同時変更に対応
    category_ids = []
    host_ids = []
    host_categoryids.each do |hostid, categoryids|
      host_ids << hostid
      category_ids |= categoryids
    end
    count_custom_category_hosts = self.joins(:sidfmd_category)
                                      .where(:sidfmd_categories => { :id => category_ids, :is_system => false })
                                      .where.not(:sidfmd_categories_hosts => { :sidfmd_host_id => host_ids })
                                      .group(:sidfmd_category_id)
                                      .count(:sidfmd_host_id)
    host_categoryids.each do |hostid, categoryids|
      categoryids.each do |cat_id|
        if count_custom_category_hosts[cat_id.to_i] then
          count_custom_category_hosts[cat_id.to_i] += 1
        else
          count_custom_category_hosts[cat_id.to_i] = 1
        end
      end
    end
    ret = SidfmdCategory.where(:id => category_ids, :is_system => false).order('LOWER(name COLLATE "C")').select{|cat|
      cat.hostlim and cat.hostlim < (count_custom_category_hosts[cat.id] ? count_custom_category_hosts[cat.id] : 1)
    }
  end

  #指定したカテゴリ属するホストのidsを返す
  def self.category_host_ids(category_id:, role_admin:, user_host_ids:)
    hosts = self.hosts(category_id)
    unless role_admin then
      hosts = hosts.where(:sidfmd_host_id => user_host_ids)
    end
    host_ids = hosts.pluck(:sidfmd_host_id)
  end

end
