#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdGrpFilter < ActiveRecord::Base
  belongs_to :sidfmd_category

  # フィルタ使用可能な画面(keyword)一覧
  KEYWORD_HOSTS = 'hosts'
  KEYWORD_HOST = 'host'
  KEYWORD_VULS = 'vulnerabilities'
  KEYWORD_VUL = 'vulnerability'

  KEYWORDS = [
    KEYWORD_HOSTS,
    KEYWORD_HOST,
    KEYWORD_VULS,
    KEYWORD_VUL
  ]

  #更新・登録で使うデータ
  def data
    {
      :type => 'group',
      :id => self.id,
      :name => self.name,
      :sidfmd_category_id => self.sidfmd_category_id,
      :keyword => self.keyword,
      :filters => JSON.parse(self.filters)
    }
  end

  def self.get_filters(keyword, sidfmd_category_ids = nil)
    ret = []
    rows = self.where(:keyword => keyword).order('LOWER(name COLLATE "C")').order(:id)
    if sidfmd_category_ids then
      rows = rows.where('sidfmd_category_id IS NULL OR sidfmd_category_id IN (?)', sidfmd_category_ids)
    end
    rows.each do |row|
      begin
        ret << row.data
      rescue JSON::ParserError => e
        log_exception(e)
      end
    end
    ret
  end
  
end
