/* coding: utf-8 */
/* Copyright (C) 2020 SofTek Systems, Inc. */

//ローカルプロダクト登録画面
function BootStrapTableCellStyle(value, row, index, field) {
    switch (field){
        case 'chkbox':
            return { classes: 'chk-box' };
        case 'priority':
            return { classes: 'lproduct-priority' };
        case 'name':
            return { classes: 'lproduct-name text-left' };
        case 'versions':
            return { classes: 'lproduct-version text-left' };
        case 'eol_date':
            return { classes: 'eol-date' };
        case 'status':
            return { classes: 'lproduct-status' };
        case 'edit':
            return { classes: 'lproduct-edit' };
        default:
            return { classes: '' };
    }
}

function BootStrapTableRowStyle(row, index){
    return {
        classes: 'list_alternative'
    }
}

$(function() {
    //インポートエラーがある場合は、エラー内容を見せるために、画面の上部にスクロール位置を設定する。
    if( $.cookie('import_err') ){
        $('html,body').animate({ scrollTop: 0 }, '1');
    }
    $.removeCookie('import_err');
    const summaryTableId = 'lproducts-summary-table';
    const summaryTableElm = $('#' + summaryTableId);
    const pageName = 'lproducts';

    //プロダクトの編集
    const enableEditLproduct = () => {
        $('#' + summaryTableId + ' [data-edit="lproduct"]').on('click', function(){
            var button = $(this);
            var prod_id = button.data('lproductid');
            var modify_path = $('#lproducts_modify_path').val();
            $('#form-lproduct').attr('action', modify_path);
            $('#LproductModal-title').text('プロダクト情報の編集: ' + button.data('lproductname'));
            $('#execute-submit-form-lproduct').text('変更').attr({
                'data-method': 'modify',
                'data-prodid': prod_id
            }).prop('disabled', false);
            reset_LproductModal();
            $('#lproduct-id-for-edit').val(prod_id);
            $('#lproduct-name-field').val(button.data('lproductname'));
            $('#select-lproduct-status').val(button.data('lproductstatus') ? 'enable' : 'disable');
            var versions_data = button.data('versions');
            make_priority_options(versions_data, true, true);
            $('#select-lproduct-priority').find('option[value="' + button.data('priority') + '"]')
                                          .prop('selected', true);
            $('#select-versions-priority').find('option:last').prop('selected', true);
            $('#LproductModal-versions').val(JSON.stringify(versions_data));
            if(versions_data.length === 0){
                append_no_version_message();
            }else{
                append_version_to_versions_list(versions_data);
            }
            var eol_date = button.data('eoldate');
            $('#lproduct-eol-date-field').val(eol_date ? eol_date : '');
            $('#LproductModal').modal('show');
        });
    };

    //プロダクトの削除
    const enableDeleteLproduct = () => {
        $(`#${summaryTableId} .delete-lproduct`).on('click', function(){
            var lprod_id = $(this).attr('data-lprodid');
            var all_prod_names_with_id = JSON.parse($('#all-product-names-with-id').val());
            var prod_name = all_prod_names_with_id[lprod_id];
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                    additionalAlertMessage: '削除に失敗しました。'
                });
                if( roleOrCategoryError ) return;
                var path = $('#sidfm_lproducts_get_issue_related_lprod_ids_path').val();
                $.post(path, {'lprod_ids': JSON.stringify([ lprod_id ])}, function(issue_related_lproduct_ids){
                    var msg = '「' + prod_name + '」を削除します。'
                    if( issue_related_lproduct_ids.includes(parseInt(lprod_id)) ){
                        msg += '\n※「' + prod_name + '」は、ローカル脆弱性に関連しているため、削除すると関連するローカル脆弱性の脆弱性チケットの対応状況(ステータス)が次のホストの脆弱性同期時に変更される可能性があります。';
                    }
                    msg += 'よろしいですか?';
                    var conf = confirm(msg);
                    if( conf ){
                        const delPath = $('#form-delete-' + lprod_id).attr('action');
                        $('body').append(create_loading_window());
                        $.ajax({
                            url: $('#sidfm_internal_api_check_related_rules_path').val(),
                            type: 'POST',
                            data: { 'page_name': 'lproduct', 'delete_lproduct_ids': JSON.stringify([lprod_id]), 'action_idx': 'delete' },
                        }).done(function(data){
                            const paramsData = { delete_id: lprod_id };
                            if( Object.keys(data['rule_messages']).length > 0 ){
                                $('#submit-related-rules-item-form').attr({
                                    'data-params': JSON.stringify(paramsData),
                                    'data-path': delPath
                                });
                                $('#checking-window').remove();
                                display_related_rules(data);
                            }else{
                                dynamicSummaryTable.executeProcess({
                                    paramsData,
                                    path: delPath,
                                    additionalAlertMessage: '削除に失敗しました。'
                                });
                            }
                        });
                        //$('#form-delete-' + lprod_id).submit();
                    }
                });
            })();
        });
        return false;
    };

    //サマリー表のバージョン名押下でフィルタ
    const FilterVersionLink = (clickedElm, filters, _) => {
        const searchVersion = clickedElm.attr('data-version');
        filters['search_versions'] = searchVersion;
        $('#search_version_field').val(searchVersion);
    };

    const enableFilterVersionLink = () => {
         summaryTableFilter.enableFilterClick(
            `#${summaryTableId} .search-versions-by-version`,
            'search_versions',
            FilterVersionLink
        );
    };

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
    const bindSummaryTableRowEvents = () => {
        enable_bootstrap_actions('#' + summaryTableId);
        enable_dropdown_balloon(summaryTableId);
        enableEditLproduct();
        enableDeleteLproduct();
        enableFilterVersionLink(); 
    };

    //サマリー表の表示データの変更に合わせて、プロダクトの一括処理に使うデータを変更
    const modifyPageContent = (data) => {
        $('#all-lproduct-size').val(data.all_lproduct_count);
        $('#all-product-names-with-id').val(JSON.stringify(data.hidden_params));
    };

    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all-lproduct-ids',
        chkboxSelector: `#${summaryTableId} .SelectLproduct`,
        allChkboxSelector: `#${summaryTableId} .allLproductSelect`,
        bulkBtnSelector: '.bulk-btn',
        modifyPageContent
    });

    //サマリー表とアクションエリアがあるページの共通の処理
    init_sidfm(); 
    sticky_control('SummaryTable', pageName);
    activate_sidebar_item('sidebar-item-lproducts-manip');
    set_style_for_ActionArea_on_filter_dropdown_event();
    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) });

    //サマリーテーブルの検索
    const summaryTableFilter = new SummaryTableFilter({dynamicSummaryTable});
    summaryTableFilter.enableFilterChangeLink('.filter-change');
    summaryTableFilter.enableFilterText(
       '#filter_lproduct_name',
       '#filter_lproduct_name_reset',
       { fieldSelector: '#search_lproduct_name_field' },
       'search_lproducts'
    ); //プロダクト名検索
    summaryTableFilter.enableFilterText(
       '#filter_version',
       '#filter_version_reset',
       { fieldSelector: '#search_version_field' },
       'search_versions'
    ); //バージョン検索

    summaryTableFilter.enableResetAllFilters('#filter-clear'); //全リセット


    const release_date_label = $('#release_date_label').val();
    const support_date_label = $('#support_date_label').val();
    const eol_date_label = $('#eol_date_label').val();
 
    //ブラウザバックだったらチェックボックスの表示を修正
    /*$.post($('#check_browser_back_path').val(), {'page_name': 'lproducts'},function(info){
        if(info === 'browser_back'){
            var is_all_checked = $('#pages_all_checked').val();
            if(is_all_checked === 'true'){
                var all_ids = JSON.parse($('#all-lproduct-ids').val());
                $('#LproductTable').find('#allSelect, .SelectLproduct').prop('checked', true);
                $('#pages_checked').val(JSON.stringify(all_ids));
            }else{
                $('#LproductTable').find('#allSelect, .SelectLproduct').prop('checked', false);
                var pages_checked = JSON.parse($('#pages_checked').val());
                var target_elm = '';
                for (var i = 0; i < pages_checked.length; i++){
                    var id = pages_checked[i];
                    target_elm += '.SelectLproduct[value="' + id + '"],';
                }
                target_elm = target_elm.slice(0, -1);
                $(target_elm).prop('checked', true);
           }
        }
    });*/
    
    //新規登録・変更modalの初期化
    function reset_LproductModal(){
        $('#lproduct-name-field, #input-version').val('');
        $('#select-lproduct-status, #select-version-status').val('enable');
        $('#versions-list-tbody > *').remove();
        $('#add-version-mark').attr('class', 'far fa-plus-square open-lproduct-add-versions-area');
        $('#lproduct-add-versions-area').hide();
        $('#execute-add-version').prop('disabled', true);
    }
    
    //バージョンが無い時に、バージョン一覧tableにメッセージを表示
    function append_no_version_message(){
        $('#versions-list-tbody').append( 
            $('<tr>').append(
                $('<td>').attr({
                    'id': 'no-version-message',
                    'colspan': 7
                }).text('登録されているバージョンはありません。')
            )
        );
    }

    //優先順位のselect要素内のoptionを作成
    function make_priority_options(versions_array = [], renew_select_lproduct = true, is_edit = false){
        $('#select-versions-priority > *').remove();
        for(var i = 0; i < versions_array.length + 1; i++){
            var op = $('<option>').val(i + 1).text(i + 1);
            $('#select-versions-priority').append(op);
        }
        if(renew_select_lproduct){
            $('#select-lproduct-priority > *').remove();
            var all_lproduct_size = parseInt($('#all-lproduct-size').val());
            var len = is_edit ? all_lproduct_size : all_lproduct_size + 1
            for(var i = 0; i < len; i++){
                var op = $('<option>').val(i + 1).text(i + 1);
                $('#select-lproduct-priority').append(op);
            }
        }
    }
    
    //バージョン一覧table内のtrを作成
    function make_version_tr(target_version_data){
        var v = target_version_data;
        var tr = $('<tr>').attr({
            'class': 'list_alternative',
            'data-versiontr': 'true',
            'data-editid': v.edit_id
        });
        var priority_td = $('<td>').attr({
            'class': 'text-center g-width-3em',
            'data-priority': v.priority,
        }).text(v.priority);
        var version_td = $('<td>').attr('class', 'text-left font-weight-bold g-text-break g-line-height-1_3').text(v.version);
        var release_date_td = $('<td>').attr('class', 'g-line-height-1_3')
                                       .append(
                                           $('<div>').attr('class', 'release-date-td').text(v.release_date ? v.release_date : '---')
                                       );
        var support_date_td = $('<td>').attr('class', 'g-line-height-1_3')
                                       .append(
                                           $('<div>').attr('class', 'support-date-td').text(v.support_date ? v.support_date : '---')
                                       );
        var is_support_end = false;
        if( v.eol_date ){
            var date_obj = new Date();
            var now = new Date(date_obj.getFullYear(), date_obj.getMonth(), date_obj.getDate());
            var eol_date = new Date(v.eol_date);
            is_support_end = now > eol_date;
        }
        var eol_date_elm = $('<span>').text(v.eol_date ? v.eol_date : '---');
        if( is_support_end ){
            eol_date_elm.addClass('text-danger');
        }
        var eol_date_td = $('<td>').attr('class', 'g-line-height-1_3')
                                   .append(
                                       $('<div>').attr('class', 'eol-date-td')
                                                 .append(eol_date_elm)
                                   );
        /*var is_support_end_td = $('<td>').attr('class', 'text-center g-line-height-1_3 p-1')
                                         .append(
                                             $('<i>').attr('class', (is_support_end ? 'fas fa-times' : 'far fa-circle'))
                                         );*/
        var status_td = $('<td>').attr('class', 'g-width-3em').text(v.flg_enable ? '有効' : '無効');
        var edit_td = $('<td>').attr('class', 'g-width-9em');
        var edit_btn = $('<button>').attr({
            'type': 'button',
            'class': 'btn btn-xs btn-edit edit-version g-mb-1 mr-1 text-nowrap',
            'data-versiondata': JSON.stringify(v),
        }).text('編集');
        var del_btn = $('<button>').attr({
             'type': 'button',
             'class': 'btn btn-xs btn-danger delete-version text-nowrap',
             'data-deleditid': v.edit_id
        }).text('削除');
        edit_td.append(
            $('<div>').attr('class', 'd-flex').append(edit_btn, del_btn)
        );
        tr.append(
            priority_td,
            version_td,
            release_date_td,
            support_date_td,
            eol_date_td,
            status_td,
            edit_td
        );
        return tr;
    }
            
    //変更modalを開いた時、登録されているバージョンがあればバージョン一覧tableにバージョンを表示
    function append_version_to_versions_list(versions_array){
        for(var i = 0; i < versions_array.length; i++){
            $('#versions-list-tbody').append(make_version_tr(versions_array[i]));
        }       
    }

    //バージョンを追加する時にバージョン一覧tableをupdateする。
    function add_new_version(version_data){
        var priority = version_data.priority;
        var versions = JSON.parse($('#LproductModal-versions').val());
        var change_priority_elms = [];
        var hit_flg = false;
        for(var i = 0; i < versions.length; i ++){
            var v_data = versions[i];
            if(v_data.priority === priority){
                hit_flg = true;
            }
            if(hit_flg){
                var elm = $('#versions-list-tbody').find('[data-priority="' + v_data.priority + '"]')
                change_priority_elms.push(elm);
                v_data.priority += 1; 
                elm.closest('[data-versiontr]').find('.edit-version')
                                               .attr('data-versiondata', JSON.stringify(v_data));
            }
        }
        $(change_priority_elms).each(function(){
            var new_priority = parseInt($(this).attr('data-priority')) + 1;
            $(this).attr('data-priority', new_priority)
                   .text(new_priority);
        });
        versions.push(version_data);
        versions.sort(function(a, b){
            if(a.priority > b.priority){
                return 1;
            }else{
                return -1;
            }
        });
        $('#LproductModal-versions').val(JSON.stringify(versions));
        var target_tr = $('#versions-list-tbody').find('[data-priority="' + (version_data.priority - 1) + '"]')
                                                 .closest('tr');
        if(target_tr.length === 1){
            target_tr.after(make_version_tr(version_data));
        }else{
            $('#versions-list-tbody').prepend(make_version_tr(version_data));
        }
        make_priority_options(versions, false);
        $('#select-versions-priority').find('option:last').prop('selected', true);
        $('#select-version-status').val('enable');
        $('#execute-add-version').prop('disabled', true);
        //bind_version_related_btn();
    }

    //バージョンの編集ボタンを押した時、変更popupを表示
    /*function display_popup_for_edit_version(button){
        var popup = $('<div>').attr({
            'id': 'popup-for-edit-verion',
            'class': 'edit-verion-popup form-group',
        }).width($('#LproductModal').find('.modal-content').width() * 0.75)
        .hide();
        var version_field = $('<div>').attr('class', 'mb-2').append(
            $('<label>').attr({
                'for': 'edit-version-input-version',
                'class': 'font-weight-bold'
            }).text('バージョン'),
            $('<input>').attr({
                'type': 'text',
                'class': 'form-control',
                'id': 'edit-version-input-version',
            })
        ); 
        var select_proproty_field = $('<div>').attr('class', 'mb-2').append(
            $('<label>').attr({ 
                'class': 'font-weight-bold'
            }).text('優先順位'),
            $('#select-versions-priority').clone()
                                          .attr('id', 'edit-version-select-priority')
        );
        var select_status_field = $('<div>').attr('class', 'mb-2').append(
            $('<label>').attr({
                'class': 'font-weight-bold'
            }).text('有効/無効'),
            $('<select>').attr({
                'id': 'edit-version-select-status',
                'class': 'form-control'
            }).append(
                $('<option>').val('enable'),
                $('<option>').val('disable')
            )
        );
        var exec_edit_btn_field = $('<div>').attr('class', 'd-flex').append(
            $('<button>').attr({
                'type': 'button',
                'id': 'execut-edit-version',
                'class': 'btn btn-sm btn-primary ml-auto'
            }).text('変更')
        );
        popup.append(
            version_field,
            select_proproty_field,
            select_status_field ,
            exec_edit_btn_field
        );
        $('body').append(popup);
        var target_offset = button.offset();
        popup.offset({
          'top': target_offset.top - popup.outerHeight(),
          'left': target_offset.left - popup.outerWidth() + button.outerWidth()
        }).show();
    }*/

    //バージョンの編集ボタンを押した時
    $(document).on('click', '.edit-version', function(){
        var button = $(this);
        var version_data = JSON.parse(button.attr('data-versiondata'));
        $('#edit-version-select-priority').remove();
        $('#edit-version-input-version').val(version_data.version);
        var select_version = $('<select>').attr({
            'id': 'edit-version-select-priority',
            'class': 'form-control form-control-sm'
        });
        for(var i = 0; i < JSON.parse($('#LproductModal-versions').val()).length; i++){
             select_version.append(
                 $('<option>').val(i + 1).text(i + 1)
             );
        }
        $('#edit-version-priority-field').append(
            select_version.val(version_data.priority)
        );
        $('#edit-version-release-date-field').val(version_data.release_date ? version_data.release_date : '');
        $('#edit-version-support-date-field').val(version_data.support_date ? version_data.support_date : '');
        $('#edit-version-eol-date-field').val(version_data.eol_date ? version_data.eol_date : '');   
        $('#edit-version-select-status').val(version_data.flg_enable ? 'enable' : 'disable');
        $('#edit-version-data').val(JSON.stringify(version_data));
        $('#execute-edit-version').prop('disabled', false);
        $('#EditVersionModal').find('.modal-title').text('バージョンの編集: ' + version_data.version);
        $('#EditVersionModal').modal('show');
    });
        
    //バージョンの削除ボタンを押したとき
    $(document).on('click', '.delete-version', function(){
        var edit_id = parseInt($(this).attr('data-deleditid'));
        $('#versions-list-tbody').find('[data-editid="' + edit_id + '"]').remove();
        if($('#versions-list-tbody').find('[data-versiontr]').length === 0){
            append_no_version_message();
        }
        var versions = JSON.parse($('#LproductModal-versions').val());
        var hit_flg = false;
        var del_idx;
        for(var i = 0; i < versions.length; i ++){
            var version_data = versions[i];
            if(hit_flg){
                var elm = $('#versions-list-tbody').find('[data-priority="' + version_data.priority + '"]');
                elm.attr('data-priority', parseInt(version_data.priority) - 1)
                   .text(parseInt(version_data.priority) - 1);
                version_data.priority -= 1;
                var edit_btn = elm.closest('[data-versiontr]').find('.edit-version');
                edit_btn.attr('data-versiondata', JSON.stringify(version_data));
            }
            if(version_data.edit_id === edit_id){
                hit_flg = true;
                var del_idx = versions.indexOf(version_data);
            }
         }
        if(del_idx != null){
            versions.splice(del_idx, 1);
        }
        make_priority_options(versions, false);
        $('#select-versions-priority option:last').prop('selected', true);
        $('#LproductModal-versions').val(JSON.stringify(versions));
           
    });

    //同名のバージョンは登録できないようにする。
    function prevent_adding_same_version_name(version_name){
        var target_versions_data = JSON.parse($('#LproductModal-versions').val());
        var is_same_version_name = false;
        for(var i = 0; i < target_versions_data.length; i ++){
            var d = target_versions_data[i];
            if(d.version == version_name){
                is_same_version_name = true;
                break;
            }
        }
        return is_same_version_name;
    }
   
    //ページネーションのパラーメータをセット
    function set_paginator_param(offset_selector, per_page_selector, order_selector, filters_selector){
        $(offset_selector).val($('#pages_offset').val());
        $(per_page_selector).val($('#pages_per_page').val());
        $(order_selector).val($('#pages_order').val());
        $(filters_selector).val($('#pages_filters').val());
    }

    //新規登録modal
    $('#create-new-lproduct').on('click', function(){
        var create_path = $('#lproducts_create_path').val();
        $('#form-lproduct').attr('action', create_path);
        $('#LproductModal-title').text('新規登録');
        $('#execute-submit-form-lproduct').text('登録').attr({
            'data-method': 'new',
            'data-prodid': 0
        }).prop('disabled', true);
        reset_LproductModal();
        append_no_version_message();
        make_priority_options();
        $('#select-lproduct-priority').find('option:last').prop('selected', true);
        $('#LproductModal-versions').val('[]');
        $('#lproduct-eol-date-field').val('');

        $('#LproductModal').modal('show');
    });
 
    //バージョンの追加
    $('#execute-add-version').on('click', function(){
        var add_version_name = $('#input-version').val().trim();
        $('#input-version').val(add_version_name);
        if(add_version_name.length === 0){
            alert('空白文字のみのバージョンは登録できません');
            return false;
        }
        var is_same_version = prevent_adding_same_version_name(add_version_name);
        if(is_same_version){
            alert('既に追加済みのバージョンです。');
            return false;
        }
        $('#no-version-message').remove();
        var product_versions = JSON.parse($('#LproductModal-versions').val());
        product_versions.sort(function(a, b){
            if(a.edit_id > b.edit_id){
                return 1;
            }else{
                return -1;
            }
        });
        var flg_enable = ($('#select-version-status option:selected').val() === 'enable' ? true : false);
        var release_date = get_date($('#version-release-date-field').val());
        var support_date = get_date($('#version-support-date-field').val());
        var eol_date = get_date($('#version-eol-date-field').val());
        var date_items = {
            [release_date_label] : release_date,
            [support_date_label] : support_date,
            [eol_date_label] : eol_date,
        };
        for( label in date_items ){
             if( date_items[label] === false ){
                 alert(label + 'は、西暦4桁、及び月と日のある日付を入力して下さい。\n※数字は半角で入力して下さい。');
                 return false;
             }
        }
        var version_data = {
            'id': 0,
            'flg_enable': flg_enable,
            'priority': parseInt($('#select-versions-priority option:selected').val()),
            'version':  add_version_name,
            'edit_id':  (product_versions.length > 0 ? product_versions[product_versions.length - 1].edit_id + 1 : 1),
            'release_date': release_date,
            'support_date': support_date,
            'eol_date': eol_date
        };
        add_new_version(version_data);
        $('#input-version, #version-release-date-field, #version-support-date-field, #version-eol-date-field').val('');
    });

    //プロダクト名が１文字以上あればプロダクト登録・変更を可能にする
    $('#lproduct-name-field').on('input', function(){
        $(this).val($(this).val().replace(/^\s+/, ''));//初めの文字が空白は許可しない
        if($(this).val().length > 0){ 
            $('#execute-submit-form-lproduct').prop('disabled', false);
        }else{
            $('#execute-submit-form-lproduct').prop('disabled', true);
        }
    });

    //バージョン名が１文字以上あればバージョン追加可能にする
    $('#input-version').on('input', function(){
        $(this).val($(this).val().replace(/^\s+/, ''));//初めの文字が空白は許可しない
        if($(this).val().length > 0){
            $('#execute-add-version').prop('disabled', false);
        }else{
            $('#execute-add-version').prop('disabled', true);
        }
    });

    //バージョン名が１文字以上あればバージョン追加可能にする(バージョン変更modal)
    $('#edit-version-input-version').on('input', function(){
        $(this).val($(this).val().replace(/^\s+/, ''));//初めの文字が空白は許可しない
        if($(this).val().length > 0){
            $('#execute-edit-version').prop('disabled', false);
        }else{
            $('#execute-edit-version').prop('disabled', true);
        }
    });

    //バージョンを追加する+ボタン・-ボタンを押した時、開閉する
    $('#toggle-add-version-area').on('click', function(){
        var mark = $('#add-version-mark');
        if(mark.hasClass('open-lproduct-add-versions-area')){
            $('#lproduct-add-versions-area').show();
            //$('#LproductModal').scrollTop($('#LproductModal').get(0).scrollHeight - $('#LproductModal').height());
            mark.attr('class', 'far fa-minus-square close-lproduct-add-versions-area');
        }else{
            $('#lproduct-add-versions-area').hide();
            //$('#LproductModal').scrollTop(0);
            mark.attr('class', 'far fa-plus-square open-lproduct-add-versions-area');
        }
    });

    function getRegistParamsData(method){
        const paramsData = {};
        const form = $('#form-lproduct');
        if( method === 'new' ){
            form.find('[name]:not([disabled])').each((idx, elm) => {
                elm = $(elm);
                switch (elm.attr('name')){
                    case 'form[lprod_name]':
                        paramsData.lprod_name = elm.val();
                        break;
                    case 'form[lprod_priority]':
                        paramsData.lprod_priority = elm.val();
                        break;
                    case 'form[eol_date]':
                        paramsData.eol_date = elm.val();
                        break;
                    case 'form[lprod_status]':
                        paramsData.lprod_status = elm.val();
                        break;
                    case 'form[versions]':
                        paramsData.versions = elm.val();
                        break;
                };
            });
        }else{ //変更
            form.find('[name]:not([disabled])').each((idx, elm) => {
                elm = $(elm);
                switch (elm.attr('name')){
                    case 'form[lprod_name]':
                        paramsData.lprod_name = elm.val();
                        break;
                    case 'form[lprod_priority]':
                        paramsData.lprod_priority = elm.val();
                        break;
                    case 'form[eol_date]':
                        paramsData.eol_date = elm.val();
                        break;
                    case 'form[lprod_status]':
                        paramsData.lprod_status = elm.val();
                        break;
                    case 'form[versions]':
                        paramsData.versions = elm.val();
                        break;
                    case 'form[lproduct_id]':
                        paramsData.lproduct_id = elm.val();
                        break;
                };
            });
        }
        return paramsData;
    }

    //プロダクトの登録・変更ボタンを押した時
    $('#execute-submit-form-lproduct').on('click', function(){
        const button = $(this);
        button.prop('disabled', true);
        var eol_date = get_date($('#lproduct-eol-date-field').val());
        if( eol_date === false ){
            alert(eol_date_label + 'は、西暦4桁、及び月と日のある日付を入力して下さい。\n※数字は半角で入力して下さい。');
            button.prop('disabled', false);
            return false;
        }
        var prod_id =  $(this).attr('data-prodid');
        var method_name = $(this).attr('data-method')
        var path = $('#sidfm_lproducts_get_issue_related_remove_version_prod_ids_path').val();
        var versions = JSON.parse($('#LproductModal-versions').val());
        var version_ids = versions.map(function(item){ return item['id'] });
        (async () => {
            const alertMessage = ((method_name === 'new' ? '登録' : '更新') + 'に失敗しました。');
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: alertMessage
            });
            if( roleOrCategoryError ) return;
            $.post(path, {'lprod_ids_with_version_ids': JSON.stringify({ [prod_id]: version_ids })}, function(issue_related_lproduct_ids){
                var conf = true;
                if( issue_related_lproduct_ids.includes(parseInt(prod_id)) ){
                    var conf = confirm('削除したバージョンはローカル脆弱性に関連しているため、関連するローカル脆弱性の脆弱性チケットの対応状況(ステータス)が、次のホストの脆弱性同期時に変更される可能性があります。\nよろしいですか?');
                }
                if( !conf ){
                    button.prop('disabled', false);
                    return false;
                }

                $('#lproduct-name-field').val($('#lproduct-name-field').val().trim());
                var path = $('#api_check_is_same_lprod_name_path').val();
                $.post(path,{
                    'method_name': method_name,
                    'prod_name': $('#lproduct-name-field').val(),
                    'prod_id': prod_id
                }, function(is_same_prod_name){
                    if(is_same_prod_name){
                        alert('既に登録済みのプロダクトです。');
                        button.prop('disabled', false);
                        return false;
                    }else{
                        //set_paginator_param('#LproductModal-pages-offset', '#LproductModal-pages-perpage', '#LproductModal-pages-order', '#LproductModal-pages-filters')
                        //$('#form-lproduct').submit();
                        $('body').append(create_loading_window());
                        $.ajax({
                            url: $('#sidfm_internal_api_check_related_rules_path').val(),
                            type: 'POST',
                            data: { 'page_name': 'lproduct', 'change_lproduct_lversion_ids': JSON.stringify({[prod_id]: version_ids}), 'action_idx': 'modify' },
                        }).done(function(data){
                            const paramsData = getRegistParamsData(method_name);
                            const actionPath = $('#form-lproduct').attr('action');
                            if( Object.keys(data['rule_messages']).length > 0 ){
                                $('#submit-related-rules-item-form').attr({
                                    'data-params': JSON.stringify(paramsData),
                                    'data-path': actionPath
                                });
                                $('#checking-window').remove();
                                display_related_rules(data);
                            }else{
                                dynamicSummaryTable.executeProcess({
                                    paramsData,
                                    path: actionPath,
                                    additionalAlertMessage: alertMessage
                                });
                            }
                            button.prop('disabled', false);
                        });
                    }
                });
            });
        })();
        return false;
    });

    //バージョンの更新
    function update_version(version_data, old_version_data){
        var modify_elms_priority = function(elms_array, num){
            $(elms_array).each(function(){
                var new_priority = parseInt($(this).attr('data-priority')) + num;
                $(this).attr('data-priority', new_priority)
                       .text(new_priority);
            }); 
        };    
        var priority = version_data.priority;
        var old_edit_id = old_version_data['edit_id'];
        $('#versions-list-tbody').find('[data-editid="' + old_edit_id + '"]').remove();
        var old_priority = old_version_data.priority
        var versions = JSON.parse($('#LproductModal-versions').val());
        versions = versions.filter(function(v){ return v.edit_id != old_version_data.edit_id });
        if(priority != old_priority){
            var is_priority_up = priority < old_priority;
            if(is_priority_up){
                var add_priority_version_elms = [];
                for(var i = 0; i < versions.length; i++){
                    v_data = versions[i];
                    if(v_data.priority < old_priority && v_data.priority >= priority){
                        var elm = $('#versions-list-tbody').find('[data-priority="' + v_data.priority + '"]');
                        add_priority_version_elms.push(elm);
                        v_data.priority += 1;
                        $('#versions-list-tbody').find('[data-editid="' + v_data.edit_id + '"]')
                                                 .find('.edit-version')
                                                 .attr('data-versiondata', JSON.stringify(v_data));
                    }
                }
                modify_elms_priority(add_priority_version_elms, 1);
            }else{
                var deduct_priority_version_elms = [];
                for(var i = 0; i < versions.length; i++){
                    v_data = versions[i];
                    if(v_data.priority > old_priority && v_data.priority <= priority){
                        var elm = $('#versions-list-tbody').find('[data-priority="' + v_data.priority + '"]');
                        deduct_priority_version_elms.push(elm);
                        v_data.priority -= 1; 
                        $('#versions-list-tbody').find('[data-editid="'+ v_data.edit_id + '"]')
                                                 .find('.edit-version')
                                                 .attr('data-versiondata', JSON.stringify(v_data));
                    }
                }
                modify_elms_priority(deduct_priority_version_elms, -1);
            }
        }
        versions.push(version_data);
        versions.sort(function(a, b){
            if(a.priority > b.priority){
                return 1;
            }else{
                return -1;
            }
        });
        $('#LproductModal-versions').val(JSON.stringify(versions));
        var target_tr = $('#versions-list-tbody').find('[data-priority="' + (version_data.priority - 1) + '"]')
                                                 .closest('tr');
        if(target_tr.length === 1){
            target_tr.after(make_version_tr(version_data));
        }else{
            $('#versions-list-tbody').prepend(make_version_tr(version_data));
        }
        //make_priority_options(versions, false);
        //bind_version_related_btn();
        
    }
    //バージョン変更modalで変更ボタンを押した時
    $('#execute-edit-version').on('click', function(){
        var mod_version_name = $('#edit-version-input-version').val().trim();
        var edit_version_data = JSON.parse($('#edit-version-data').val());
        if(mod_version_name != edit_version_data['version']){
            var is_same_version = prevent_adding_same_version_name(mod_version_name);
            if(is_same_version){
                $('#edit-version-input-version').val(mod_version_name);
                alert('既に追加済みのバージョンです。');
                return false;
            }
        }
        var priority = parseInt($('#edit-version-select-priority option:selected').val());
        var release_date = get_date($('#edit-version-release-date-field').val());
        var support_date = get_date($('#edit-version-support-date-field').val());
        var eol_date = get_date($('#edit-version-eol-date-field').val());
        var date_items = {
            [release_date_label] : release_date,
            [support_date_label] : support_date,
            [eol_date_label] : eol_date,
        };
        for( label in date_items ){
             if( date_items[label] === false ){
                 alert(label + 'は、西暦4桁、及び月と日のある日付を入力して下さい。\n※数字は半角で入力して下さい。');
                 return false;
             }
        }
        var flg_enable_val = $('#edit-version-select-status option:selected').val();
        var update_version_data = {
            'id': edit_version_data.id,
            'flg_enable': (flg_enable_val == 'enable' ? true : false),
            'priority': priority,
            'version': mod_version_name,
            'edit_id': edit_version_data.edit_id,
            'release_date': release_date,
            'support_date': support_date,
            'eol_date': eol_date 
        }
        update_version(update_version_data, edit_version_data);
        $('#EditVersionModal').modal('hide');
    });

    //新規登録・変更modalが閉じる時
    $('#LproductModal').on('hide.bs.modal', function(){
        $('#lproduct-versions-list').scrollTop(0);
    });

    //有効/無効一括変更
    $('#bulk-change-status').on('click', function(){
        $('#bulk-status-tbody > *').remove();
        var all_prod_names_with_id = JSON.parse($('#all-product-names-with-id').val());
        var checked = JSON.parse($('#pages_checked').val());
        $('#bulk-lproduct-ids').val($('#pages_checked').val());
        $('#bulk-lproduct-status-select option:first').prop('selected', true);
        $('#BulkLproductStatusModal').modal('show');
        checked.forEach(function(id){
            var prod_name = all_prod_names_with_id[id]; 
            var tr = $('<tr>');
            tr.append(
                $('<td>').attr('class', 'text-left bulk-lproduct-name-td')
                         .text(prod_name)
            );
            $('#bulk-status-tbody').append(tr);
        });
    });

    $('#execute-bulk-status-change').on('click', function(){
        const paramsData = {
            lproduct_ids: $('#bulk-lproduct-ids').val(),
            flg_enable: $('#bulk-lproduct-status-select').val(),
        };
        dynamicSummaryTable.executeProcess({
            paramsData,
            path: $('#bulk-lproduct-status-form').attr('action'),
            additionalAlertMessage: '更新に失敗しました。'
        });
        return false;
        /*set_paginator_param(
            '#BulkLproductStatusModal-pages-offset',
            '#BulkLproductStatusModal-pages-perpage',
            '#BulkLproductStatusModal-pages-order',
            '#BulkLproductStatusModal-pages-filters'
        );
        $('#bulk-lproduct-status-form').submit();*/
    });

    //プロダクト一括削除
    $('#bulk-delete-lproducts').on('click', function(){
        var checked = JSON.parse($('#pages_checked').val());
        var all_prod_names_with_id = JSON.parse($('#all-product-names-with-id').val());
        var path = $('#sidfm_lproducts_get_issue_related_lprod_ids_path').val();
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '削除に失敗しました。'
            });
            $.post(path, {'lprod_ids': JSON.stringify(checked)}, function(issue_related_lproduct_ids){
                var prod_names = []
                var issue_related_prod_names = []
                checked.forEach(function(id){
                    prod_names.push('「' + all_prod_names_with_id[id] + '」');
                    if( issue_related_lproduct_ids.includes(id) ){
                        issue_related_prod_names.push('「' + all_prod_names_with_id[id] + '」');
                    }
                });
                var msg = prod_names.join(',') + 'を削除します。'
               if( issue_related_prod_names.length > 0 ){
                    msg += '\n※' + issue_related_prod_names.join(',') + 'は、ローカル脆弱性に関連しているため、削除すると関連するローカル脆弱性の脆弱性チケットの対応状況(ステータス)が次のホストの脆弱性同期時に変更される可能性があります。';
                }
                msg += 'よろしいですか?';
                var conf = confirm(msg);
                if(conf){
                    $('body').append(create_loading_window());
                    $.ajax({
                        url: $('#sidfm_internal_api_check_related_rules_path').val(),
                        type: 'POST',
                        data: { 'page_name': 'lproduct', 'delete_lproduct_ids': JSON.stringify(checked), 'action_idx': 'bulk_delete' },
                    }).done(function(data){
                        const paramsData = { delete_ids: JSON.stringify(checked) };
                        const delPath = $('#bulk-lproducts-delete-form').attr('action');
                        if( Object.keys(data['rule_messages']).length > 0 ){
                            $('#submit-related-rules-item-form').attr({
                                'data-params': JSON.stringify(paramsData),
                                'data-path': delPath
                            });
                            $('#checking-window').remove();
                            display_related_rules(data);
                        }else{
                            dynamicSummaryTable.executeProcess({
                                paramsData,
                                path: delPath,
                                additionalAlertMessage: '削除に失敗しました。'
                            });
                        }
                    });
                    //$('#bulk-lproducts-delete-ids').val(JSON.stringify(checked));
                    //$('#bulk-lproducts-delete-form').submit(); 
                }
            });
        })();
        return false;
    });

    //インポートmodal
    $('#import-lproduct-files').on('click', function(){
        $('#modalImport').modal('show');
    });

    //インポート
    $('#submit-import-files').on('click', function(){
        var import_path = $('#sidfm_lproducts_import_path').val();
        var progress_import_path = $('#sidfm_lproducts_progress_import_path').val();
        var read_file_path = $('#sidfm_lproducts_read_import_file_path').val();
        var lprod_fd = new FormData();
        var lver_fd = new FormData();
        var lprod_file = $('#import-lproduct-file').get(0).files[0];
        var lver_file = $('#import-lversion-file').get(0).files[0];
        var lprod_data_name = 'lproduct_file';
        var lver_data_name = 'lversion_file';
        if( !lprod_file ){
            alert('ローカルプロダクト情報ファイルを選択して下さい。\n※ローカルプロダクト情報ファイルは必須です。');
            return false;
        }else{
            lprod_fd.append(lprod_data_name, lprod_file);
        }

        if( !lver_file ){
            var conf = confirm(
                'ローカルバージョン情報ファイルが選択されておりません。\n' + 
                'インポートを開始してもよろしいですか?'
            );
            if( !conf ){
                return false;
            }
        }else{
            lver_fd.append(lver_data_name, lver_file);
        }
        var progress_window = get_progress_bar();
        $('body').append(progress_window);

        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: 'インポートに失敗しました。'
            });
            if( roleOrCategoryError ) return;
            var percentage = 0;
            var total_size = 0;
            var executed_size = 0;
         
            var progress_import = function(){
                $.ajax({
                    url: progress_import_path,
                    type: 'GET',
                }).done(function(res){
                    if( res['executed_size'] > 0 ){
                        executed_size += res['executed_size'];
                        var percentage = (executed_size / total_size * 100);
                        if( percentage >= 99 && percentage < 100 ){
                            percentage = 99;
                        }else{
                            percentage = Math.round(percentage);
                        }
                        $('#progress-bar').attr('aria-valuenow', percentage).css('width', percentage + '%').text(percentage + '%');
                        progress_import();
                    }else{
                        if( res['same_prod_name_err'] ){
                            alert('ファイルのインポートに失敗しました。画面をリロードします。');
                            $.cookie('import_err', 'true');
                            $('#pages_form').submit();
                        }else{
                            setTimeout(function(){//probress barのbarが満たされるまで待つ
                                location.href = import_path;
                            }, 500);
                        }
                    }
                }).fail(function(){
                    alert('ファイルのインポートに失敗しました。画面をリロードします。');
                    location.reload();    
                });
            };
            
            //プロダクト情報ファイル読み込み
            $.ajax({
                url: read_file_path,
                type: 'POST',
                data: lprod_fd,
                processData: false,
                contentType: false,
            }).done(function(res){
                if( res['err'] ){
                    alert('ファイルのインポートに失敗しました。画面をリロードします。');
                    $.cookie('import_err', 'true');
                    location.reload();
                }else{
                    if( res['total_size'] ){
                        total_size = res['total_size'];
                    }
                    //バージョン情報ファイル読み込み
                    if( lver_file ){
                        $.ajax({
                            url: read_file_path,
                            type: 'POST',
                            data: lver_fd,
                            processData: false,
                            contentType: false,
                        }).done(function(res){
                            if( res['err'] ){
                                alert('ファイルのインポートに失敗しました。画面をリロードします。');
                                $.cookie('import_err', 'true');
                                location.reload();
                            }else{
                                progress_import();    
                            }
                        }).fail(function(){
                            alert('ファイルのインポートに失敗しました。画面をリロードします。');
                            location.reload();
                        });
                    }else{
                        progress_import();
                    }
                }
            }).fail(function(){
                alert('ファイルのインポートに失敗しました。画面をリロードします。');
                location.reload();
            });
        })();
    });

    //エクスポート
    $('#export-lproducts, #export-lversions').on('click', function(){
        var is_err = false;
        var message = '';
        var progress_window = get_progress_bar('get');
        $('body').append(progress_window);
        var export_target = ($(this).attr('id') === 'export-lproducts' ? 'lproducts' : 'lversions');
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: 'ファイルのエクスポートに失敗しました'
            });
            if( roleOrCategoryError ) return;
            $('#export-target-tag').val(export_target);
            var export_path = $('#sidfm_lproducts_export_path').val();
            var checked = $('#pages_checked').val();
            if( JSON.parse(checked).length === 0 ){
                checked = $('#all-lproduct-ids').val();    
            }
            var total_size = 0;
            var executed_size = 0;
            var export_file = $.ajax({
                url: export_path ,
                type: 'POST',
                data: { 'checked': checked, 'process': 'start_export', 'target':  export_target },
            }).done(function(res){
                if( res['err'] ){
                    is_err = true;
                    message = res['message']
                }else{
                    total_size = res['total_size'];
                }
            }).fail(function(){
                message = 'ファイルのエクスポートに失敗しました';
                location.reload();
            });

            $.when(export_file).done(function(){
                if( is_err ){
                    alert(message + '画面をリロードします。');
                    location.reload();
                    return false;
                }
                
                var is_end = false;
                var progress_export = function(){
                    $.ajax({
                        url: export_path ,
                        type: 'POST',
                        data: { 'process': 'progress_export', 'executed_size': executed_size, 'target':  export_target },
                    }).done(function(res){
                        if( res['err'] ){
                            is_err = true;
                            message = res['message'];
                            alert(message + '画面をリロードします。');
                            location.reload();
                        }else if( res['executed_size'] == 0 ){
                            is_end = true;
                            $('#form-export').submit();
                            $('#progress-bar-label').text('ファイル送信中...');
                            var timer = setInterval(function(){
                                if( $.cookie('export_flg') ){
                                    setTimeout(function(){//probress barのbarが満たされるまで待つ
                                        $.cookie('export_flg',"",{path:"/",expires:-1});
                                        $('#progress-window').remove();
                                    },500);
                                    clearInterval(timer);
                                }
                            },10);
                        }else{
                            executed_size += res['executed_size'];
                            var percentage = (executed_size / total_size * 100);
                            if( percentage >= 99 && percentage < 100 ){
                                percentage = 99;
                            }else{
                                percentage = Math.round(percentage);
                            }
                            $('#progress-bar').attr('aria-valuenow', percentage).css('width', percentage + '%').text(percentage + '%');
                        }
                        if( !is_err && !is_end ){
                            progress_export();
                        }
                    }).fail(function(){
                        is_err = true
                        message = 'ファイルのエクスポートに失敗しました。';
                    });
                };

                progress_export();

                if( is_err ){
                    alert(message + '画面をリロードします。');
                    location.reload();
                }
            });
        })();
    });

    //ルールの処理を実行
    $('#submit-related-rules-item-form').on('click', function(){
        const button = $(this);
        const paramsData = JSON.parse(button.attr('data-params'));
        const path = button.attr('data-path');
        dynamicSummaryTable.executeProcess({
            paramsData,
            path,
            additionalAlertMessage: '更新に失敗しました。'
        });
        return false; 
    });
});
